import java.util.*;

/**
 * Represents a complete binary tree in which each
 * node stores a value that can be compared.
 */
public class CompleteBinaryTree<E extends Comparable<E>> {

  ArrayList<E> data; // data stored at the nodes

  /**
   * Constructs an empty binary tree.
   */
  public CompleteBinaryTree() {
    data = new ArrayList<>();
  }

  /**
   * Constructs a complete binary tree, assuming the input
   * items are ordered from top-to-bottom and then left-to-right
   * within each level.
   * 
   * @param items - items to stores at the nodes.
   */
  public CompleteBinaryTree(E[] items) {
    data = new ArrayList<>();
    for (E value : items) {
      data.add(value);
    }
  }

  /**
   * @return number of nodes in the binary tree.
   */
  public int size() {
    return data.size();
  }

  /**
   * @return index of the left child of node i.
   */
  public static int left(int i) {
    return 2 * i + 1;
  }

  /**
   * @return index of the right child of node i.
   */
  public static int right(int i) {
    return 2 * (i + 1);
  }

  /**
   * @return index of the parent of node i.
   */
  public static int parent(int i) {
    // TODO
    return 0;
  }

  /**
   * @return whether this complete binary tree represents a max-heap.
   */
  public boolean isMaxHeap() {
    // TODO
    return false;
  }

  /**
   * @return a String representation of this tree.
   */
  public String toString() {
    return toStringHelper("", 0);
  }

  /**
   * Returns a string representation of the subtree.
   * 
   * @param padding - spacing and vertical bars to print before the node.
   * @param node    - the root index of the subtree to print.
   * @return appended line to print for this node (and child nodes).
   */
  public String toStringHelper(String padding, int index) {
    if (index >= data.size())
      return "";

    String result = padding + "└──(" + data.get(index).toString() + ")" + "\n";

    padding += "│   ";
    result += toStringHelper(padding, left(index));
    result += toStringHelper(padding, right(index));

    return result;
  }

  public static void main(String[] args) {
    CompleteBinaryTree<Integer> tree = new CompleteBinaryTree<>();
    tree = new CompleteBinaryTree<>(new Integer[] { 16, 14, 10, 8, 7, 9, 3, 2, 4, 1 });
    System.out.println(tree);
    System.out.println(tree.isMaxHeap());
  }
}
